#******************************************************************************/
#*      Main event navigator window for use with Athena 12.0.6                */
#*                                                                            */
#*                                           Sebastian Boeser, 30.05.07       */
#******************************************************************************/

#Define a global ref to our algorithm
global HepMCBrowserAlg

#Global list of avaible MC Event Collection storegate keys
MCEvtColls = ["GEN_EVENT","TruthEvent","GEN_AOD"]

#-- ROOT classes
import ROOT
from ROOT import TGTextButton
from ROOT import TGLabel
from ROOT import TGMainFrame
from ROOT import TGTab
from ROOT import TGTransientFrame
from ROOT import TGHProgressBar
from ROOT import TGMsgBox
from ROOT import TControlBar
from ROOT import TGCheckButton
from ROOT import TBrowser
from ROOT import TGLayoutHints
from ROOT import TGHorizontalFrame
from ROOT import TGVerticalFrame
from ROOT import TGGroupFrame
from ROOT import TGComboBox
from ROOT import TGTextEntry
from ROOT import TGNumberEntryField
from ROOT import TGNumberFormat

from ROOT import TCanvas
from ROOT import TLatex

#-- ROOT globals
from ROOT import SetOwnership
from ROOT import gClient
from ROOT import gSystem

#-- ROOT constants
from ROOT import NULL
from ROOT import kMainFrame
from ROOT import kVerticalFrame
from ROOT import kHorizontalFrame
from ROOT import kLHintsLeft
from ROOT import kLHintsRight
from ROOT import kLHintsTop
from ROOT import kLHintsBottom
from ROOT import kLHintsExpandX
from ROOT import kLHintsExpandY
from ROOT import kLHintsCenterX
from ROOT import kLHintsCenterY
from ROOT import kMBIconStop
from ROOT import kMBIconQuestion
from ROOT import kMBIconExclamation
from ROOT import kMBIconAsterisk
from ROOT import kMBYes
from ROOT import kMBNo
from ROOT import kMBOk
from ROOT import kMBApply
from ROOT import kMBRetry
from ROOT import kMBIgnore
from ROOT import kMBCancel
from ROOT import kMBClose
from ROOT import kMBDismiss

#STL from ROOT
from ROOT import std

# Load most important namespace
# -- note that libraries have been loaded before in HepMCLoadLibs.py
from ROOT import HepMC

#-- Using StatusCode objects instead of Py:Athena int version
from ROOT import StatusCode

#-- Athena imports
import AthenaPython.PyAthena as PyAthena
from AthenaCommon.AppMgr import ServiceMgr
from AthenaCommon.AppMgr import theApp
from AthenaCommon.Logging import logging
mlog = logging.getLogger('HepMCBrowser')


class HepMCBrowserFrame ( PyAthena.Alg ):
  """Open the HepMCBrowser Window to navigate through events"""
   
  ## Constructor
  def __init__( self, name = "HepMCBrowser", **keywords ):
    # Set name in keywords
    keywords['name'] = name
    # Init PyAthena.Alg itself
    super(HepMCBrowserFrame,self).__init__(**keywords)
    # the GUI Window
    self.fMainFrame=NULL
    # the Browser Canvas
    self.cBrowser=NULL
    # the max number of events
    self.fNEntries=-1

    #create a global ref to myself
    global HepMCBrowserAlg
    HepMCBrowserAlg = self

  ## Initilization - here we generate the window
  def initialize( self ):
    
    #Get hold of the StoreGateSvc
    self.StoreGate = PyAthena.py_svc('StoreGateSvc')
    if self.StoreGate is None:
      mlog.error('Problem retrieving StoreGateSvc!')
      return StatusCode(StatusCode.FAILURE)
    
    #the main Frame
    self.fMainFrame = TGMainFrame(gClient.GetRoot(),10,10,
                                  kMainFrame | kHorizontalFrame);
    SetOwnership(self.fMainFrame,False);
    self.fMainFrame.SetWindowName("HepMC browser");
    self.fMainFrame.Resize(250,350);

    #create some layout hints
    self.fExpLay = TGLayoutHints(kLHintsLeft | kLHintsTop |
                                 kLHintsExpandX | kLHintsExpandY,2,2,2,2)
    self.fButFixLay = TGLayoutHints(kLHintsLeft | kLHintsCenterY |
                                  kLHintsExpandY,2,2,2,2)
    self.fButRightLay = TGLayoutHints(kLHintsRight | kLHintsCenterY |
                                  kLHintsExpandY,2,2,2,2)
    self.fTopExpHLay = TGLayoutHints(kLHintsCenterX | kLHintsTop |
                                 kLHintsExpandX ,2,2,2,2)
    self.fBotExpHLay = TGLayoutHints(kLHintsCenterX | kLHintsBottom |
                                 kLHintsExpandX ,2,2,2,2)
    self.fRightLay = TGLayoutHints(kLHintsCenterY | kLHintsRight |
                                   kLHintsExpandY ,2,2,2,2)
    self.fLeftLay = TGLayoutHints(kLHintsCenterY | kLHintsLeft |
                                 kLHintsExpandY ,2,2,2,2)
    self.fExpLeftLay = TGLayoutHints( kLHintsLeft |
                                 kLHintsExpandX ,2,2,2,2)

    #Add a tab
    self.fTab = TGTab(self.fMainFrame);
    self.fMainFrame.AddFrame(self.fTab,self.fExpLay);
    
    #Add an event control tab
    self.fVertFrame = self.fTab.AddTab("Event");

    #Add a horizontal button frame at the top
    self.fHorButFrame = TGHorizontalFrame(self.fVertFrame,
                                          100,100,kHorizontalFrame);
    self.fVertFrame.AddFrame(self.fHorButFrame, self.fTopExpHLay );

    #Add a first event button
    self.fButFirst = TGTextButton(self.fHorButFrame,"|<<");
    self.fHorButFrame.AddFrame(self.fButFirst, self.fButFixLay );
    self.fButFirst.SetCommand('TPython::Exec("HepMCBrowserAlg.seekByEntryNumber(0)");');

    #Add a previous event button
    self.fButPrev = TGTextButton(self.fHorButFrame,"<<");
    self.fHorButFrame.AddFrame(self.fButPrev, self.fButFixLay );
    self.fButPrev.SetCommand('TPython::Exec("HepMCBrowserAlg.seekPrevious()");');

    #Add the event counter
    self.fLblEntryNr = TGLabel(self.fHorButFrame,"Entry Nr.:");
    self.fHorButFrame.AddFrame(self.fLblEntryNr, self.fExpLay);

    self.fEntryNr = TGNumberEntryField(self.fHorButFrame, -1, -1,
      TGNumberFormat.kNESInteger, TGNumberFormat.kNEAAnyNumber,
      TGNumberFormat.kNELLimitMin, -1)
    self.fHorButFrame.AddFrame(self.fEntryNr, self.fExpLay);

    #Add a next button
    self.fButNext = TGTextButton(self.fHorButFrame,">>");
    self.fHorButFrame.AddFrame(self.fButNext, self.fButFixLay );
    self.fButNext.SetCommand('TPython::Exec("HepMCBrowserAlg.seekNext()");');

    #Seek for a certain entry number
    self.fButGo = TGTextButton(self.fHorButFrame,"Go");
    self.fHorButFrame.AddFrame(self.fButGo, self.fButFixLay );
    self.fButGo.SetCommand('TPython::Exec("HepMCBrowserAlg.seekByEntryNumber()");');

    #Add a frame for the event collection
    self.fHorCollFrame = TGHorizontalFrame(self.fVertFrame,
                                          100,100,kHorizontalFrame);
    self.fVertFrame.AddFrame(self.fHorCollFrame, self.fTopExpHLay );
    #Add a label
    self.fLblColl = TGLabel(self.fHorCollFrame,"Event Collection: ");
    self.fHorCollFrame.AddFrame(self.fLblColl, self.fLeftLay);
    #Add a list box to select the MC collection
    self.fMCCollList = TGComboBox(self.fHorCollFrame);
    self.fHorCollFrame.AddFrame(self.fMCCollList, self.fExpLay );
    #Add all the possible MC Event Collection StoreGate keys
    for key in MCEvtColls:
      self.fMCCollList.AddEntry(key, MCEvtColls.index(key));
    #Select first entry by default
    self.fMCCollList.Select(0)

    #Add the storeGate event info frame
    self.fSGInfoFrame = TGGroupFrame(self.fVertFrame,"ATLAS Event info");
    self.fVertFrame.AddFrame(self.fSGInfoFrame,self.fTopExpHLay);

    #Add the run number
    self.fLblRunNr = TGLabel(self.fSGInfoFrame,"Run number: ?");
    self.fSGInfoFrame.AddFrame(self.fLblRunNr, self.fLeftLay);


    #Add the event number
    self.fEvtNrFrame = TGHorizontalFrame(self.fSGInfoFrame,
                                          100,100,kHorizontalFrame);
    self.fSGInfoFrame.AddFrame(self.fEvtNrFrame, self.fTopExpHLay );

    self.fLblEvtNr = TGLabel(self.fEvtNrFrame,"Event number:");
    self.fEvtNrFrame.AddFrame(self.fLblEvtNr, self.fLeftLay);
    
    self.fEventNr = TGNumberEntryField(self.fEvtNrFrame, -1, -1,
      TGNumberFormat.kNESInteger, TGNumberFormat.kNEAAnyNumber,
      TGNumberFormat.kNELLimitMin, -1)
    self.fEvtNrFrame.AddFrame(self.fEventNr, self.fTopExpHLay);
    #Add a search button
    self.fButSearchEvent = TGTextButton(self.fEvtNrFrame,"Search");
    self.fEvtNrFrame.AddFrame(self.fButSearchEvent, self.fButRightLay );
    self.fButSearchEvent.SetCommand('TPython::Exec("HepMCBrowserAlg.seekByEventNumber(None,\'ATLAS\')");')

    #Add the HepMC event info frame
    self.fInfoFrame = TGGroupFrame(self.fVertFrame,"HepMC Event info");
    self.fVertFrame.AddFrame(self.fInfoFrame,self.fExpLay);

    #Add the HEPMC event number
    self.fHepMCEvtNrFrame = TGHorizontalFrame(self.fInfoFrame,
                                          100,100,kHorizontalFrame);
    self.fInfoFrame.AddFrame(self.fHepMCEvtNrFrame, self.fTopExpHLay );

    self.fLblHepMCEvtNr = TGLabel(self.fHepMCEvtNrFrame,"Event number:");
    self.fHepMCEvtNrFrame.AddFrame(self.fLblHepMCEvtNr, self.fLeftLay);
    
    self.fHepMCEventNr = TGNumberEntryField(self.fHepMCEvtNrFrame, -1, -1,
      TGNumberFormat.kNESInteger, TGNumberFormat.kNEAAnyNumber,
      TGNumberFormat.kNELLimitMin, -1)
    self.fHepMCEvtNrFrame.AddFrame(self.fHepMCEventNr, self.fTopExpHLay);
    #Add a search button
    self.fButSearchHepMCEvent = TGTextButton(self.fHepMCEvtNrFrame,"Search");
    self.fHepMCEvtNrFrame.AddFrame(self.fButSearchHepMCEvent, self.fButRightLay );
    self.fButSearchHepMCEvent.SetCommand('TPython::Exec("HepMCBrowserAlg.seekByEventNumber(None,\'HEPMC\')");')

    #Add the particle counters
    self.fLblParticles = TGLabel(self.fInfoFrame,"Nr. of particles: ?");
    self.fInfoFrame.AddFrame(self.fLblParticles, self.fLeftLay);
    #Add the vertex counters
    self.fLblVertices = TGLabel(self.fInfoFrame,"Nr. of vertices: ?");
    self.fInfoFrame.AddFrame(self.fLblVertices, self.fLeftLay);
    #Add alphaQED
    self.fLblAlphaQED = TGLabel(self.fInfoFrame,"Alpha-QED: ?");
    self.fInfoFrame.AddFrame(self.fLblAlphaQED, self.fLeftLay);
    #Add the vertex counters
    self.fLblAlphaQCD = TGLabel(self.fInfoFrame,"Alpha-QCD: ?");
    self.fInfoFrame.AddFrame(self.fLblAlphaQCD, self.fLeftLay);

    #Add an exit button
    self.fButExit = TGTextButton(self.fVertFrame,"Exit");
    self.fVertFrame.AddFrame(self.fButExit, self.fBotExpHLay );
    self.fButExit.SetCommand('TPython::Exec("HepMCBrowserAlg.quit()");');
    
    #Add a filter control tab
    self.fFilterFrame = self.fTab.AddTab("Filter");

    #Add a print button
    self.fButPrint = TGTextButton(self.fFilterFrame,"Print event");
    self.fFilterFrame.AddFrame(self.fButPrint, self.fBotExpHLay );
    self.fButPrint.SetCommand('TPython::Exec("HepMCBrowserAlg.printEvent()");')

    #Add the Pythia frame
    self.fPythiaFrame = TGGroupFrame(self.fFilterFrame,"Generators");
    self.fFilterFrame.AddFrame(self.fPythiaFrame,self.fTopExpHLay);

    #Hadronization button
    self.fHadBut = TGCheckButton(self.fPythiaFrame,"Remove fragmentation");
    self.fPythiaFrame.AddFrame(self.fHadBut, self.fExpLeftLay );
    self.fHadBut.SetDown(False);

    #Add the Geant frame
    self.fGeantFrame = TGGroupFrame(self.fFilterFrame,"Geant");
    self.fFilterFrame.AddFrame(self.fGeantFrame,self.fTopExpHLay);

    #Geant removal button
    self.fGeantBut = TGCheckButton(self.fGeantFrame,"Remove GEANT");
    self.fGeantFrame.AddFrame(self.fGeantBut, self.fExpLeftLay );
    self.fGeantBut.SetDown(False);

    #Add the Vertex finder frame
    self.fVertexFrame = TGGroupFrame(self.fFilterFrame,"Signal vertex");
    self.fFilterFrame.AddFrame(self.fVertexFrame,self.fExpLay);

    #Vertex finder button
    self.fVertexBut = TGCheckButton(self.fVertexFrame,"Search for signal vertex");
    self.fVertexFrame.AddFrame(self.fVertexBut, self.fExpLeftLay );
    self.fVertexBut.SetDown(False);
    
    #Add the in-particle list
    self.fInParticleFrame = TGHorizontalFrame(self.fVertexFrame,
                                          100,100,kHorizontalFrame);
    self.fVertexFrame.AddFrame(self.fInParticleFrame, self.fTopExpHLay );

    self.fLblInParticle = TGLabel(self.fInParticleFrame,"In-particle IDs:");
    self.fInParticleFrame.AddFrame(self.fLblInParticle, self.fLeftLay);
    
    self.fInParticle = TGTextEntry(self.fInParticleFrame);
    self.fInParticleFrame.AddFrame(self.fInParticle, self.fExpLay);
    self.fInParticle.SetToolTipText("Comma-seperated list of particle IDs of"
       "required in-coming particles, e.g. \"21,21\""); 
    
    #Add the out-particle list
    self.fOutParticleFrame = TGHorizontalFrame(self.fVertexFrame,
                                          100,100,kHorizontalFrame);
    self.fVertexFrame.AddFrame(self.fOutParticleFrame, self.fTopExpHLay );

    self.fLblOutParticle = TGLabel(self.fOutParticleFrame,"Out-particle IDs:");
    self.fOutParticleFrame.AddFrame(self.fLblOutParticle, self.fLeftLay);
    
    self.fOutParticle = TGTextEntry(self.fOutParticleFrame);
    self.fOutParticleFrame.AddFrame(self.fOutParticle, self.fExpLay);
    self.fOutParticle.SetToolTipText("Comma-seperated list of particle IDs of"
       "required out-going particles, e.g. \"11,-11\""); 

    #Do the layout and draw
    self.fMainFrame.Layout();
    self.fMainFrame.MapSubwindows();
    self.fMainFrame.MapWindow();
    
    #Add a canvas
    self.ResetCanvas();

    #Show welcome message
    self.ShowStartupScreen();

    #finally, try to load a PDT 
#    iopdt=HepMC.IO_PDG_ParticleDataTable("PDG_table_2006.mc");
#    self.pdt = iopdt.read_particle_data_table();
#    self.pdt.make_antiparticles_from_particles();

    #return
    return StatusCode(StatusCode.SUCCESS)

  def ShowStartupScreen ( self ):

    #Some welcome message
    fLblWelcome = TLatex(.3,.7,"Welcome to HepMC Browser")
    fLblWelcome.Draw();
    SetOwnership(fLblWelcome,False)

    #Show a nice startup event
    m=HepMC.HepFourVector(0,0,0,0)
    g1 = HepMC.VisualParticle(m,21); g1.SetShow();
    g2 = HepMC.VisualParticle(m,21); g2.SetShow();
    v1 = HepMC.VisualVertex(.3,.5); v1.add_particle_in(g1);
    v2 = HepMC.VisualVertex(.3,.3); v2.add_particle_in(g2);
    t1 = HepMC.VisualParticle(m,6); t1.SetShow();
    t2 = HepMC.VisualParticle(m,6); t2.SetShow();
    t3 = HepMC.VisualParticle(m,6); t3.SetShow();
    v3 = HepMC.VisualVertex(.4,.4); 
    v2.add_particle_out(t1); v2.add_particle_out(t2);
    v1.add_particle_out(t3); v1.add_particle_in(t1);
    v3.add_particle_in(t3); v3.add_particle_in(t2);
    v4 = HepMC.VisualVertex(.5,.4); 
    h1 = HepMC.VisualParticle(m,25); h1.SetShow();
    v3.add_particle_out(h1); v4.add_particle_in(h1);
    z1 = HepMC.VisualParticle(m,23); z1.SetShow();
    z2 = HepMC.VisualParticle(m,23); z2.SetShow();
    v4.add_particle_out(z1); v4.add_particle_out(z2);
    v5 = HepMC.VisualVertex(.6,.5); v5.add_particle_in(z1);
    v6 = HepMC.VisualVertex(.6,.3); v6.add_particle_in(z2);
    e1 = HepMC.VisualParticle(m,6); e1.SetShow();
    e2 = HepMC.VisualParticle(m,6); e2.SetShow();
    e3 = HepMC.VisualParticle(m,6); e3.SetShow();
    e4 = HepMC.VisualParticle(m,6); e4.SetShow();
    v5.add_particle_out(e1); v5.add_particle_out(e2);
    v6.add_particle_out(e3); v6.add_particle_out(e4);

    # All HepMCVisual objects not owned by root
    SetOwnership(g1,False);
    SetOwnership(g2,False);
    SetOwnership(t1,False);
    SetOwnership(t2,False);
    SetOwnership(t3,False);
    SetOwnership(h1,False);
    SetOwnership(v1,False);
    SetOwnership(v2,False);
    SetOwnership(v3,False);
    SetOwnership(v4,False);
    SetOwnership(z1,False);
    SetOwnership(z2,False);
    SetOwnership(v5,False);
    SetOwnership(v6,False);
    SetOwnership(e1,False);
    SetOwnership(e2,False);
    SetOwnership(e3,False);
    SetOwnership(e4,False);
    v3.Draw();

    #Update canvas
    self.cBrowser.Update();

  ##Generate a new canvas if necessary
  def ResetCanvas( self ):
    if not self.cBrowser :
      #Initialize a canvas to draw the event
      self.cBrowser = TCanvas("cBrowser","Welcome to HepMC Browser",900,600)
      self.cBrowser.Draw()
      SetOwnership(self.cBrowser,False)

    #Clear Browser window
    self.cBrowser.Clear();

  ##Update the storeGate event information in the browser frame
  def UpdateEventInfo( self, evtinfo = None ):

    #Default in case there is no EventInfo object
    RunNum = "Run Number: ?" 
    EvtNum = -1

    #check if we found an EventID object
    evtid = self.getEventID()
    if evtid:
      #Just show run number and event number
      RunNum = "Run Number: %i" % evtid.run_number()
      EvtNum = evtid.event_number()

    #Update text fields
    self.fLblRunNr.SetText(RunNum)
    self.fEventNr.SetIntNumber(EvtNum)
    
    #Update layout
    self.fSGInfoFrame.Layout();
    self.fSGInfoFrame.MapSubwindows();
    self.fSGInfoFrame.MapWindow();
  
  ##Update the HepMC event information in the browser frame
  def UpdateHepMCEventInfo( self, evt ):
    
    #Set the text fields
    self.fHepMCEventNr.SetIntNumber(evt.event_number())
    NPart = "Nr of particles: %i" % evt.particles_size()
    self.fLblParticles.SetText(NPart)
    NVert = "Nr of vertices: %i" % evt.vertices_size()
    self.fLblVertices.SetText(NVert)
    AQED = "Alpha-QED: %f" % evt.alphaQED()
    self.fLblAlphaQED.SetText(AQED)
    AQCD = "Alpha-QCD: %f" % evt.alphaQCD()
    self.fLblAlphaQCD.SetText(AQCD)

    #Update layout
    self.fInfoFrame.Layout();
    self.fInfoFrame.MapSubwindows();
    self.fInfoFrame.MapWindow();
  
  ## Popup a Message box
  def MsgBox( self , title = "" , message = "" ):
    #Defaul icon
    icon = kMBIconAsterisk
    #check for message type
    if title == "Error":
      icon = kMBIconStop
    if title == "Warning":
      icon = kMBIconStop
    #Show the message box
    msg = TGMsgBox(gClient.GetRoot(),self.fMainFrame,title,message,icon,kMBOk)
    #python will destroy this
    ROOT.SetOwnership(msg,False)

  ## Wrap ApplicationMgr seek method which does not return StatusCode
  def seek ( self, entrynr ):
    #Use theApp seeker
    retval = theApp.seek(entrynr)
    #Check for nonetype return
    if retval is None:
      return StatusCode(StatusCode.FAILURE)
    else :
      return StatusCode(retval)

  ## getEventInfo block from storegate
  def getEventID( self ):

    #Get the event info block
    EvtInfo = self.StoreGate.retrieve('EventInfo','McEventInfo')

    if EvtInfo :
      return EvtInfo.event_ID();
    else :
      return None

  ##Get an MC events
  def getMCEvent( self, StoreGateKey = "GEN_AOD", silent = False ):
    
    #Get the Monte Carlo Events
    MCEvents = self.StoreGate.retrieve('McEventCollection',StoreGateKey)

    #Test if there is a collection
    if not MCEvents:
      if not silent:
        self.MsgBox("Error",
                    "Collection \""+StoreGateKey+"\" not found in this event!")
      return NULL

    #Get first event
    Evt = MCEvents[0]
  
    #Test if there was an event
    if not Evt:
      if not silent:
        self.MsgBox("Error",
                    "Empty collection - no event found!")
      return NULL

    #return event if everything is alright...
    return Evt
  
  #get the StoreGate key from the combo box
  def getStoreGateKey( self ):
    return self.fMCCollList.GetSelectedEntry().GetText().GetString()

  #return current event
  def getCurEntry ( self ):
    return self.fEntryNr.GetIntNumber();

  #try to find end of file
  def getNEntries ( self ):

    #see if we had calculated this before
    if self.fNEntries > -1: return self.fNEntries
    
    # seek forward in increasing steps until end of file is reached
    def expSeekFwd ( start ):
      dStep = 1
      #double step size in every step
      while self.seek(start+dStep).isSuccess() : dStep *=2
      #return the last okay one
      return start+(dStep/2)

    #Poup a window to show what we are doing
    self.fMainStat = TGTransientFrame(ROOT.gClient.GetRoot(),self.fMainFrame,200,35)
    ROOT.SetOwnership(self.fMainStat,False)
    #add a description
    self.fStatLbl = TGLabel(self.fMainStat, "Probing number of entries...");
    self.fMainStat.AddFrame(self.fStatLbl, self.fExpLay);
    
    #Show window
    self.fMainStat.Layout();
    self.fMainStat.MapWindow();
    self.fMainStat.MapSubwindows();
    ROOT.gClient.ProcessEventsFor(self.fMainStat);

    #Now start looking at beginning
    start = 0;
    end = expSeekFwd(start)
    #seek until there was only one step
    while end - start > 1 :
      #remember new start
      start = end
      #seek again
      end = expSeekFwd(start)

    #Unshow window when done
    self.fMainStat.UnmapWindow();
#   self.fMainStat.Delete()
    

    #store the last okay on
    self.fNEntries=end
    #be verbose  
    mlog.debug("Found %i events in input collection" % self.fNEntries)
    #and return it
    return self.fNEntries

  ##Seeking forward/backward: wrap theApp - seekers
  
  def seekNext( self ):
    self.seekByEntryNumber(self.getCurEntry()+1)

  def seekPrevious( self ):
    self.seekByEntryNumber(self.getCurEntry()-1)

  def seekByEntryNumber( self, entrynr = None ):
    #get from entry number field if necessary
    if entrynr == None:
      entrynr = self.getCurEntry();
    #for some reason seek fails on last event --> check with event number
    if entrynr > self.getNEntries():
      self.MsgBox("Error","Only "+str(self.getNEntries())+" entries available!")
      return
    #dry test run first
    if self.seek(entrynr).isSuccess() :
      theApp.seekEvent(entrynr)
    else : 
      self.MsgBox("Error", "Entry "+str(entrynr)+" does not exist!");
  
  ##Seek for an event by ATLAS/HepMC event number
  def seekByEventNumber( self, evtnr = None, mode="ATLAS" ):
   
    if not ( ( mode == "ATLAS" ) or ( mode == "HEPMC" ) ):
      log.error("Valid search modes are 'ATLAS' and 'HEPMC' ")
      return

    if evtnr == None:
      if mode == "ATLAS" :
        evtnr = self.fEventNr.GetIntNumber()
      else :
        evtnr = self.fHepMCEventNr.GetIntNumber()
    
    #we do not want to run all the algorithms while seeking
    # -- store old algorithm list
    OldAlgList = theApp.TopAlg

    # clear TopAlg List
    theApp.TopAlg = []

    # create a window to show progress
    self.fMainProg = TGTransientFrame(ROOT.gClient.GetRoot(),self.fMainFrame,400,50)
    ROOT.SetOwnership(self.fMainProg,False)
    #add a description
    descr = "Searching for "+mode+" event number "+str(evtnr)
    if mode == "HEPMC": descr += " in "+str(self.getStoreGateKey())
    descr += " ..."
    self.fProgLbl = TGLabel(self.fMainProg, descr);
    self.fMainProg.AddFrame(self.fProgLbl, self.fExpLay);
    
    #add the progress bar itself
    self.fProg = TGHProgressBar(self.fMainProg,TGHProgressBar.kStandard);
    self.fProg.SetRange(0,self.getNEntries())
    self.fMainProg.AddFrame(self.fProg, self.fExpLay);
    
    #Show window
    self.fMainProg.Layout();
    self.fMainProg.MapWindow();
    self.fMainProg.MapSubwindows();
    ROOT.gClient.ProcessEventsFor(self.fMainProg);

    #loop over events and check event number
    EntryNr = 0; found = False;
    while  EntryNr <= self.getNEntries():
      
      #Show progress
      self.fProg.SetPosition(EntryNr)
      ROOT.gClient.ProcessEventsFor(self.fMainProg);

      #Go to this event
      if theApp.seekEvent(EntryNr).isSuccess():

        #check if looking for ATLAS event number
        if mode == "ATLAS":

          #get the event ID
          EvtID = self.getEventID();
  
          #check if event is valid
          if EvtID:
            
            #test if it has the right evtnr
            if EvtID.event_number() == evtnr :
              found = True
              break
                   
        else :
          #Get the event record
          Evt = self.getMCEvent(self.getStoreGateKey(), silent = True)

          #check if event is valid
          if Evt:
            
            #test if it has the right evtnr
            if Evt.event_number() == evtnr :
              found = True
              break

      #Goto next event
      EntryNr += 1

    #clear the window
    self.fMainProg.UnmapWindow();
#    self.fMainProg.Delete();
   
    #reinstall old algorithm list
    theApp.TopAlg = OldAlgList 

    #if the event was found, 
    if found:
      # execute the AlgList
      theApp.seekEvent(EntryNr);
    else :
      # show a message box
      text = "Event Nr. "+str(evtnr)+" not found"
      if mode == "HEPMC": text += " in "+self.getStoreGateKey()+" collection" 
      text += "!"
      self.MsgBox("Warning",text)

  ## get a std::vector of incoming particles
  def getVectorFromString ( self , string ):
    
    #declare the empty vector
    particles = std.vector(int)()
    
    #get a comma separated list of entries 
    list = string.split(",")

    #Loop over entries
    for entry in list:
      #check for empty fields`
      if len(entry.strip(" ")) != 0 : 
        #check if can be converted to an int
        try :
          intv = int(entry)
        except ValueError:
          self.MsgBox("Error",entry+" is not a valid particle ID ! - Ignored!")
        else :
          particles.push_back(int(entry))

    #return the particle vector
    return particles
  
  ##print the current event using HepMC::GenEvent::print()
  def printEvent( self ):

    #Get the event record
    Evt = self.getMCEvent(self.getStoreGateKey(), silent = True);
    
    #python has overloaded print() on HepMC 
    #need to visualize the event
    VisEvt = HepMC.Visualizer().Event(Evt);

    #print it
    VisEvt.print_event();
    
  ##Execute -- give a new event to the browser
  def execute( self ):

    #Reset browser window
    self.ResetCanvas();
    
    #Update event number counter
    self.fEntryNr.SetIntNumber(theApp.curEvent())

    #Get MC event
    Evt = self.getMCEvent(self.getStoreGateKey())
    if not Evt: 
      mlog.warning("No event found!!!");
      return True;

    #Update browser window title
    title = "HepMC Browser - Event "+'%(evt)i'%{'evt':Evt.event_number()}
    self.cBrowser.SetTitle(title)

    #run the various filters
    filter = HepMC.VisualFilter();
    
    #check for hadronisation removal
    if self.fHadBut.IsDown():
      filter.RemovePythiaFragmentation(Evt);
    
    #check for hadronisation removal
    if self.fGeantBut.IsDown():
      filter.RemoveGeant(Evt);

    #check for vertex finder
    if self.fVertexBut.IsDown():
      #get in- and out-particle lists
      inl = self.getVectorFromString(self.fInParticle.GetText())
      outl = self.getVectorFromString(self.fOutParticle.GetText())
      #Try to find vertex
      if filter.FindSignalProcess(Evt,inl,outl):
        mlog.info("Found signal process vertex!")
      else :
        mlog.info("Signal process vertex not found!")

    #Visualize Event and draw it
    VisEvt = HepMC.Visualizer().Event(Evt);
    SetOwnership(VisEvt,False);
    #Give it the PDT
#    VisEvt.SetPDT(self.pdt);
    VisEvt.Draw();

    self.UpdateEventInfo( self.getEventID() );
    self.UpdateHepMCEventInfo( Evt );

    #Update browser Window
    self.cBrowser.Update();
    
    #return
    return StatusCode(StatusCode.SUCCESS)

  ##Finalize -- close the window
  def finalize( self ):
    frame = self.fMainFrame
    if frame :
      frame.CloseWindow()
    return StatusCode(StatusCode.SUCCESS)

  ##quit -- exit athena and the application
  def quit( self ):
    print "\tByeBye!";
    theApp.exit(0)
    #gSystem.Exit(0);
    return
